/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.apache.zeppelin.rest.message.gson;

import com.google.gson.JsonElement;
import com.google.gson.JsonObject;
import com.google.gson.JsonSerializationContext;
import com.google.gson.JsonSerializer;
import org.apache.commons.lang3.exception.ExceptionUtils;

import java.lang.reflect.Type;
import javax.ws.rs.WebApplicationException;

public class ExceptionSerializer implements JsonSerializer<Exception> {

  @Override
  public JsonElement serialize(
      Exception e, Type type, JsonSerializationContext jsonSerializationContext) {
    JsonObject jsonObject = new JsonObject();
    jsonObject.addProperty("exception", e.getClass().getSimpleName());
    jsonObject.addProperty("message", e.getMessage());
    jsonObject.addProperty("stacktrace", ExceptionUtils.getStackTrace(e));

    if (e instanceof WebApplicationException) {
      jsonObject.addProperty("status", ((WebApplicationException) e).getResponse().getStatus());
    }

    return jsonObject;
  }
}
